@@ -34,6 +34,8 @@ from aea_test_autonomy.fixture_helpers import (  # noqa: F401
     abci_host,
     abci_port,
     flask_tendermint,
+    ipfs_daemon,
+    ipfs_domain,
     tendermint_port,
 )
 
