@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,11 +18,13 @@
 # ------------------------------------------------------------------------------
 
 """Fuzzy tests for valory/abci connection"""
-
+import os
 import platform
 
 import pytest
+from hypothesis import settings
 
+from packages.valory.connections.abci import CI
 from packages.valory.connections.abci.tests.test_fuzz.base import BaseFuzzyTests
 from packages.valory.connections.abci.tests.test_fuzz.mock_node.channels.grpc_channel import (
     GrpcChannel,
@@ -32,6 +34,11 @@ from packages.valory.connections.abci.tests.test_fuzz.mock_node.channels.tcp_cha
 )
 
 
+running_on_ci = os.getenv(CI)
+if running_on_ci:
+    settings.load_profile(CI)
+
+
 @pytest.mark.skipif(
     platform.system() == "Windows",
     reason="<IocpProactor overlapped#=1175 result#=0> is running after closing for ... seconds (windows_events.py:871)",
