@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,9 +18,7 @@
 # ------------------------------------------------------------------------------
 
 """Test the base.py module of the skill."""
-
-# pylint: skip-file
-
+import dataclasses
 import datetime
 import logging
 import re
@@ -28,6 +26,7 @@ import shutil
 from abc import ABC
 from contextlib import suppress
 from copy import copy, deepcopy
+from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
 from time import sleep
@@ -81,9 +80,6 @@ from packages.valory.skills.abstract_round_abci.base import (
 )
 from packages.valory.skills.abstract_round_abci.base import _logger as default_logger
 from packages.valory.skills.abstract_round_abci.base import get_name
-from packages.valory.skills.abstract_round_abci.serializer import (
-    DictProtobufStructSerializer,
-)
 from packages.valory.skills.abstract_round_abci.test_tools.abci_app import (
     AbciAppTest,
     ConcreteBackgroundRound,
@@ -98,12 +94,20 @@ from packages.valory.skills.abstract_round_abci.test_tools.abci_app import (
 from packages.valory.skills.abstract_round_abci.tests.conftest import profile_name
 
 
+# pylint: skip-file
+
+
 settings.load_profile(profile_name)
 
 
 PACKAGE_DIR = Path(__file__).parent.parent
 
 
+DUMMY_CONCRETE_BACKGROUND_PAYLOAD = ConcreteBackgroundRound.payload_class(
+    sender="sender"
+)
+
+
 @pytest.fixture(scope="session", autouse=True)
 def hypothesis_cleanup() -> Generator:
     """Fixture to remove hypothesis directory after tests."""
@@ -121,6 +125,7 @@ class PayloadEnum(Enum):
     B = "B"
     C = "C"
     DUMMY = "DUMMY"
+    TOO_BIG_TO_FIT_IN_HERE = "TOO_BIG_TO_FIT_IN_HERE"
 
     def __str__(self) -> str:
         """Get the string representation."""
@@ -141,65 +146,38 @@ class BasePayload(BaseTxPayload, ABC):
     """Base payload class for testing."""
 
 
+@dataclass(frozen=True)
 class PayloadA(BasePayload):
     """Payload class for payload type 'A'."""
 
-    transaction_type = PayloadEnum.A
-
 
+@dataclass(frozen=True)
 class PayloadB(BasePayload):
     """Payload class for payload type 'B'."""
 
-    transaction_type = PayloadEnum.B
-
 
+@dataclass(frozen=True)
 class PayloadC(BasePayload):
     """Payload class for payload type 'C'."""
 
-    transaction_type = PayloadEnum.C
-
 
+@dataclass(frozen=True)
 class PayloadD(BasePayload):
     """Payload class for payload type 'D'."""
 
-    transaction_type = PayloadEnumB.A
-
 
+@dataclass(frozen=True)
 class DummyPayload(BasePayload):
     """Dummy payload class."""
 
-    transaction_type = PayloadEnum.DUMMY
-
-    def __init__(self, sender: str, dummy_attribute: int, **kwargs: Any) -> None:
-        """Initialize a dummy payload.
-
-        :param sender: the sender address
-        :param dummy_attribute: a dummy attribute
-        :param kwargs: the keyword arguments
-        """
-        super().__init__(sender, **kwargs)
-        self._dummy_attribute = dummy_attribute
-
-    @property
-    def dummy_attribute(self) -> int:
-        """Get the dummy_attribute."""
-        return self._dummy_attribute
-
-    @property
-    def data(self) -> Dict:
-        """Get the data."""
-        return dict(dummy_attribute=self.dummy_attribute)
+    dummy_attribute: int
 
 
-class TooBigPayload(BaseTxPayload, ABC):
+@dataclass(frozen=True)
+class TooBigPayload(BaseTxPayload):
     """Base payload class for testing."""
 
-    transaction_type = PayloadEnum.A
-
-    @property
-    def data(self) -> Dict:
-        """Get the data"""
-        return dict(dummy_field="0" * 10 ** 7)
+    dummy_field: str = "0" * 10 ** 7
 
 
 class ObjectImitator:
@@ -218,12 +196,17 @@ class ObjectImitator:
 def test_base_tx_payload() -> None:
     """Test BaseTxPayload."""
 
-    payload = BasePayload(sender="sender")
+    payload = PayloadA(sender="sender")
+    object.__setattr__(payload, "round_count", 9)
     new_payload = payload.with_new_id()
 
-    assert payload.sender == new_payload.sender
-    assert payload.id_ != new_payload.id_
-    payload.round_count = 1
+    assert not payload == new_payload
+    payload_data, new_payload_data = payload.json, new_payload.json
+    assert not payload_data.pop("id_") == new_payload_data.pop("id_")
+    assert payload_data == new_payload_data
+    with pytest.raises(dataclasses.FrozenInstanceError):
+        payload.round_count = 1  # type: ignore
+    object.__setattr__(payload, "round_count", 1)
     assert payload.round_count == 1
     assert type(hash(payload)) == int
 
@@ -251,7 +234,7 @@ def test_abstract_round_instantiation_without_attributes_raises_error() -> None:
 
         class MyRoundBehaviourC(AbstractRound):
             synchronized_data_class = MagicMock()
-            allowed_tx_type = MagicMock()
+            payload_class = MagicMock()
 
 
 class TestTransactions:
@@ -259,7 +242,7 @@ class TestTransactions:
 
     def setup(self) -> None:
         """Set up the test."""
-        self.old_value = copy(_MetaPayload.transaction_type_to_payload_cls)
+        self.old_value = copy(_MetaPayload.registry)
 
     def test_encode_decode(self) -> None:
         """Test encoding and decoding of payloads."""
@@ -290,6 +273,16 @@ class TestTransactions:
         actual = expected.decode(expected.encode())
         assert expected == actual
 
+    def test_encode_too_big_payload(self) -> None:
+        """Test encode of a too big payload."""
+        sender = "sender"
+        payload = TooBigPayload(sender)
+        with pytest.raises(
+            ValueError,
+            match=f"{type(payload)} must be smaller than {MAX_READ_IN_BYTES} bytes",
+        ):
+            payload.encode()
+
     def test_encode_too_big_transaction(self) -> None:
         """Test encode of a too big transaction."""
         sender = "sender"
@@ -327,7 +320,7 @@ class TestTransactions:
 
     def teardown(self) -> None:
         """Tear down the test."""
-        _MetaPayload.transaction_type_to_payload_cls = self.old_value
+        _MetaPayload.registry = self.old_value
 
 
 @mock.patch(
@@ -336,64 +329,30 @@ class TestTransactions:
 def test_verify_transaction_negative_case(*_mocks: Any) -> None:
     """Test verify() of transaction, negative case."""
     transaction = Transaction(MagicMock(sender="right_sender", json={}), "")
-    with pytest.raises(SignatureNotValidError, match="signature not valid."):
+    with pytest.raises(
+        SignatureNotValidError, match="Signature not valid on transaction: .*"
+    ):
         transaction.verify("")
 
 
+@dataclass(frozen=True)
+class SomeClass(BaseTxPayload):
+    """Test class."""
+
+    content: Dict
+
+
 @given(
     dictionaries(
         keys=text(),
         values=one_of(floats(allow_nan=False, allow_infinity=False), booleans()),
     )
 )
-def test_dict_serializer_is_deterministic(obj: Any) -> None:
+def test_payload_serializer_is_deterministic(obj: Any) -> None:
     """Test that 'DictProtobufStructSerializer' is deterministic."""
-    obj_bytes = DictProtobufStructSerializer.encode(obj)
-    for _ in range(100):
-        assert obj_bytes == DictProtobufStructSerializer.encode(obj)
-        assert obj == DictProtobufStructSerializer.decode(obj_bytes)
-
-
-class TestMetaPayloadUtilityMethods:
-    """Test _MetaPayload private utility methods."""
-
-    def setup(self) -> None:
-        """Set up the test."""
-        self.old_value = copy(_MetaPayload.transaction_type_to_payload_cls)
-
-    def test_meta_payload_validate_tx_type(self) -> None:
-        """
-        Test _MetaPayload._validate_transaction_type utility method.
-
-        First, it registers a class object with a transaction type name into the
-        _MetaPayload map from transaction type name to classes.
-        Then, it tries to validate a new insertion with the same transaction type name
-        but different class object. This will raise an error.
-        """
-        tx_type_name = "transaction_type"
-        tx_cls_1 = MagicMock(__name__="name_1")
-        tx_cls_2 = MagicMock(__name__="name_2")
-        _MetaPayload.transaction_type_to_payload_cls[tx_type_name] = tx_cls_1
-
-        with pytest.raises(ValueError):
-            _MetaPayload._validate_transaction_type(tx_type_name, tx_cls_2)
-
-    def test_get_field_positive(self) -> None:
-        """Test the utility class method "_get_field", positive case"""
-        expected_value = 42
-        result = _MetaPayload._get_field(
-            MagicMock(field_name=expected_value), "field_name"
-        )
-        return result == expected_value
-
-    def test_get_field_negative(self) -> None:
-        """Test the utility class method "_get_field", negative case"""
-        with pytest.raises(ValueError):
-            _MetaPayload._get_field(MagicMock, "field_name")
-
-    def teardown(self) -> None:
-        """Tear down the test."""
-        _MetaPayload.transaction_type_to_payload_cls = self.old_value
+    obj_ = SomeClass(sender="", content=obj)
+    obj_bytes = obj_.encode()
+    assert obj_ == BaseTxPayload.decode(obj_bytes)
 
 
 def test_initialize_block() -> None:
@@ -1010,7 +969,7 @@ class TestBaseSynchronizedData:
 
     def test_properties(self) -> None:
         """Test several properties"""
-        participants = {"b", "a"}
+        participants = ["b", "a"]
         randomness_str = (
             "3439d92d58e47d342131d446a3abe264396dd264717897af30525c98408c834f"
         )
@@ -1040,7 +999,7 @@ class TestBaseSynchronizedData:
             )
         )
         assert self.base_synchronized_data.period_count == 0
-        assert base_synchronized_data.all_participants == participants
+        assert base_synchronized_data.all_participants == frozenset(participants)
         assert base_synchronized_data.sorted_participants == ["a", "b"]
         assert abs(base_synchronized_data.keeper_randomness - randomness_value) < 1e-10
         assert base_synchronized_data.most_voted_randomness == randomness_str
@@ -1066,7 +1025,7 @@ class TestAbstractRound:
 
     def setup(self) -> None:
         """Set up the tests."""
-        self.known_payload_type = ConcreteRoundA.allowed_tx_type
+        self.known_payload_type = ConcreteRoundA.payload_class
         self.participants = {"a", "b"}
         self.base_synchronized_data = BaseSynchronizedData(
             db=AbciAppDB(setup_data=dict(participants=[self.participants]))
@@ -1081,7 +1040,7 @@ class TestAbstractRound:
 
         class MyConcreteRound(AbstractRound):
 
-            allowed_tx_type = MagicMock()
+            payload_class = MagicMock()
             synchronized_data_class = MagicMock()
             payload_attribute = MagicMock()
 
@@ -1102,7 +1061,7 @@ class TestAbstractRound:
         class MyConcreteRound(AbstractRound):
             # here round_id is missing
             # ...
-            allowed_tx_type = MagicMock()
+            payload_class = MagicMock()
             synchronized_data_class = MagicMock()
             payload_attribute = MagicMock()
 
@@ -1119,18 +1078,18 @@ class TestAbstractRound:
         my_concrete_round = MyConcreteRound(MagicMock(), MagicMock())
         assert my_concrete_round.round_id == "my_concrete_round"
 
-    def test_must_set_allowed_tx_type(self) -> None:
-        """Test that the 'allowed_tx_type' must be set in concrete classes."""
+    def test_must_set_payload_class_type(self) -> None:
+        """Test that the 'payload_class' must be set in concrete classes."""
 
         with pytest.raises(
-            AbstractRoundInternalError, match="'allowed_tx_type' not set on .*"
+            AbstractRoundInternalError, match="'payload_class' not set on .*"
         ):
 
             class MyConcreteRound(AbstractRound):
 
                 synchronized_data_class = MagicMock()
                 payload_attribute = MagicMock()
-                # here allowed_tx_type is missing
+                # here payload_class is missing
                 # ...
 
                 def end_block(self) -> Optional[Tuple[BaseSynchronizedData, EventType]]:
@@ -1142,12 +1101,12 @@ class TestAbstractRound:
                 def process_payload(self, payload: BaseTxPayload) -> None:
                     pass
 
-    def test_check_allowed_tx_type_with_previous_round_transaction(self) -> None:
-        """Test check 'allowed_tx_type'."""
+    def test_check_payload_type_with_previous_round_transaction(self) -> None:
+        """Test check 'check_payload_type'."""
 
         class MyConcreteRound(AbstractRound):
 
-            allowed_tx_type = "allowed_tx_type"
+            payload_class = BaseTxPayload
             synchronized_data_class = MagicMock()
             payload_attribute = MagicMock()
 
@@ -1163,19 +1122,17 @@ class TestAbstractRound:
         with pytest.raises(LateArrivingTransaction), mock.patch.object(
             default_logger, "debug"
         ) as mock_logger:
-            MyConcreteRound(
-                MagicMock(), MagicMock(), "previous_transaction"
-            ).check_allowed_tx_type(
-                MagicMock(payload=MagicMock(transaction_type="previous_transaction"))
+            MyConcreteRound(MagicMock(), MagicMock(), BaseTxPayload).check_payload_type(
+                MagicMock(payload=BaseTxPayload("dummy"))
             )
             mock_logger.assert_called()
 
-    def test_check_allowed_tx_type(self) -> None:
-        """Test check 'allowed_tx_type'."""
+    def test_check_payload_type(self) -> None:
+        """Test check 'check_payload_type'."""
 
         class MyConcreteRound(AbstractRound):
 
-            allowed_tx_type = None
+            payload_class = None
             synchronized_data_class = MagicMock()
             payload_attribute = MagicMock()
 
@@ -1192,7 +1149,7 @@ class TestAbstractRound:
             TransactionTypeNotRecognizedError,
             match="current round does not allow transactions",
         ):
-            MyConcreteRound(MagicMock(), MagicMock()).check_allowed_tx_type(MagicMock())
+            MyConcreteRound(MagicMock(), MagicMock()).check_payload_type(MagicMock())
 
     def test_synchronized_data_getter(self) -> None:
         """Test 'synchronized_data' property getter."""
@@ -1203,37 +1160,36 @@ class TestAbstractRound:
         """Test 'check_transaction' method, with unknown payload type."""
         tx_type = "unknown_payload"
         tx_mock = MagicMock()
-        tx_mock.payload.transaction_type = tx_type
+        tx_mock.payload_class = tx_type
         with pytest.raises(
             TransactionTypeNotRecognizedError,
-            match=f"request '{tx_type}' not recognized",
+            match="request '.*' not recognized",
         ):
             self.round.check_transaction(tx_mock)
 
     def test_check_transaction_known_payload(self) -> None:
         """Test 'check_transaction' method, with known payload type."""
         tx_mock = MagicMock()
-        tx_mock.payload.transaction_type = self.known_payload_type
+        tx_mock.payload = self.known_payload_type(sender="dummy")
         self.round.check_transaction(tx_mock)
 
     def test_process_transaction_negative_unknown_payload(self) -> None:
         """Test 'process_transaction' method, with unknown payload type."""
-        tx_type = "unknown_payload"
         tx_mock = MagicMock()
-        tx_mock.payload.transaction_type = tx_type
+        tx_mock.payload = object
         with pytest.raises(
             TransactionTypeNotRecognizedError,
-            match=f"request '{tx_type}' not recognized",
+            match="request '.*' not recognized",
         ):
             self.round.process_transaction(tx_mock)
 
     def test_process_transaction_negative_check_transaction_fails(self) -> None:
         """Test 'process_transaction' method, with 'check_transaction' failing."""
         tx_mock = MagicMock()
-        tx_mock.payload.transaction_type = "payload_a"
+        tx_mock.payload = object
         error_message = "transaction not valid"
         with mock.patch.object(
-            self.round, "check_allowed_tx_type", side_effect=ValueError(error_message)
+            self.round, "check_payload_type", side_effect=ValueError(error_message)
         ):
             with pytest.raises(ValueError, match=error_message):
                 self.round.process_transaction(tx_mock)
@@ -1241,7 +1197,7 @@ class TestAbstractRound:
     def test_process_transaction_positive(self) -> None:
         """Test 'process_transaction' method, positive case."""
         tx_mock = MagicMock()
-        tx_mock.payload.transaction_type = "payload_a"
+        tx_mock.payload = BaseTxPayload(sender="dummy")
         self.round.process_transaction(tx_mock)
 
     def test_check_majority_possible_raises_error_when_nb_participants_is_0(
@@ -1744,13 +1700,7 @@ class TestAbciApp:
     @mock.patch.object(ConcreteBackgroundRound, "check_transaction")
     @pytest.mark.parametrize(
         "transaction",
-        [
-            mock.MagicMock(
-                payload=MagicMock(
-                    transaction_type=ConcreteBackgroundRound.allowed_tx_type
-                )
-            )
-        ],
+        [mock.MagicMock(payload=DUMMY_CONCRETE_BACKGROUND_PAYLOAD)],
     )
     def test_check_transaction_for_background_round(
         self,
@@ -1765,13 +1715,7 @@ class TestAbciApp:
     @mock.patch.object(ConcreteBackgroundRound, "process_transaction")
     @pytest.mark.parametrize(
         "transaction",
-        [
-            mock.MagicMock(
-                payload=MagicMock(
-                    transaction_type=ConcreteBackgroundRound.allowed_tx_type
-                )
-            )
-        ],
+        [mock.MagicMock(payload=DUMMY_CONCRETE_BACKGROUND_PAYLOAD)],
     )
     def test_process_transaction_for_background_round(
         self,
@@ -2264,18 +2208,33 @@ class TestRoundSequence:
                 result=background_round_result[0],
             )
 
-    def test_reset_state(self) -> None:
+    @pytest.mark.parametrize("restart_from_round", (ConcreteRoundA, MagicMock()))
+    def test_reset_state(self, restart_from_round: AbstractRound) -> None:
         """Tests reset_state"""
-        restart_from_round = ConcreteRoundA
         round_count, reset_index = 1, 1
         with mock.patch.object(
             self.round_sequence,
             "_reset_to_default_params",
         ) as mock_reset:
-            self.round_sequence.reset_state(
-                restart_from_round, round_count, reset_index
-            )
-            mock_reset.assert_called()
+            transition_fn = self.round_sequence.abci_app.transition_function
+            round_id = restart_from_round.auto_round_id()
+            if restart_from_round in transition_fn:
+                self.round_sequence.reset_state(round_id, round_count, reset_index)
+                mock_reset.assert_called()
+            else:
+                round_ids = {cls.auto_round_id() for cls in transition_fn}
+                with pytest.raises(
+                    ABCIAppInternalError,
+                    match=re.escape(
+                        "internal error: Cannot reset state. The Tendermint recovery parameters are incorrect. "
+                        "Did you update the `restart_from_round` with an incorrect round id? "
+                        f"Found {round_id}, but the app's transition function has the following round ids: "
+                        f"{round_ids}.",
+                    ),
+                ):
+                    self.round_sequence.reset_state(
+                        restart_from_round.auto_round_id(), round_count, reset_index
+                    )
 
     def test_reset_to_default_params(self) -> None:
         """Tests _reset_to_default_params."""
@@ -2315,7 +2274,7 @@ def test_meta_abci_app_when_final_round_not_subclass_of_degenerate_round() -> No
     class FinalRound(AbstractRound):
         """A round class for testing."""
 
-        allowed_tx_type = MagicMock()
+        payload_class = MagicMock()
         synchronized_data_class = MagicMock()
         payload_attribute = MagicMock()
 
