@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 import json
 import logging
+from dataclasses import asdict
 from typing import Any, Dict, cast
 from unittest import mock
 from unittest.mock import MagicMock
@@ -66,6 +67,8 @@ from packages.valory.skills.abstract_round_abci.handlers import (
     Transaction,
     exception_to_info_msg,
 )
+from packages.valory.skills.abstract_round_abci.models import TendermintRecoveryParams
+from packages.valory.skills.abstract_round_abci.test_tools.rounds import DummyRound
 
 
 def test_exception_to_info_msg() -> None:
@@ -375,7 +378,7 @@ class TestTendermintHandler:
 
     def test_handle_no_addresses_retrieved_yet(self, caplog: LogCaptureFixture) -> None:
         """Test handle request no registered addresses"""
-        performative = TendermintMessage.Performative.REQUEST
+        performative = TendermintMessage.Performative.GET_GENESIS_INFO
         message = TendermintMessage(performative)  # type: ignore
         message.sender = "Alice"
         with self.mocked_registered_addresses({}):
@@ -389,7 +392,7 @@ class TestTendermintHandler:
         self, caplog: LogCaptureFixture
     ) -> None:
         """Test handle response sender not in registered addresses"""
-        performative = TendermintMessage.Performative.RESPONSE
+        performative = TendermintMessage.Performative.GENESIS_INFO
         message = TendermintMessage(performative, info="info")  # type: ignore
         message.sender = "Alice"
         self.handler.handle(message)
@@ -397,9 +400,9 @@ class TestTendermintHandler:
         assert log_message in caplog.text
 
     # request
-    def test_handle_request(self, caplog: LogCaptureFixture) -> None:
-        """Test handle request"""
-        performative = TendermintMessage.Performative.REQUEST
+    def test_handle_get_genesis_info(self, caplog: LogCaptureFixture) -> None:
+        """Test handle request for genesis info"""
+        performative = TendermintMessage.Performative.GET_GENESIS_INFO
         message = TendermintMessage(performative)  # type: ignore
         self.context.agent_address = message.sender = self.agent_name
         with self.mocked_registered_addresses(self.dummy_validator_config):
@@ -409,10 +412,10 @@ class TestTendermintHandler:
 
     # response
     def test_handle_response_invalid_addresses(self, caplog: LogCaptureFixture) -> None:
-        """Test handle response invalid address"""
+        """Test handle response for genesis info with invalid address."""
         validator_config = self.dummy_validator_config
         validator_config[self.agent_name]["hostname"] = "random"
-        performative = TendermintMessage.Performative.RESPONSE
+        performative = TendermintMessage.Performative.GENESIS_INFO
         info = json.dumps(validator_config[self.agent_name])
         message = TendermintMessage(performative, info=info)  # type: ignore
         self.context.agent_address = message.sender = self.agent_name
@@ -421,9 +424,9 @@ class TestTendermintHandler:
         log_message = self.handler.LogMessages.failed_to_parse_address.value
         assert log_message in caplog.text
 
-    def test_handle_response_valid_addresses(self, caplog: LogCaptureFixture) -> None:
-        """Test handle response valid address"""
-        performative = TendermintMessage.Performative.RESPONSE
+    def test_handle_genesis_info(self, caplog: LogCaptureFixture) -> None:
+        """Test handle response for genesis info with valid address"""
+        performative = TendermintMessage.Performative.GENESIS_INFO
         info = json.dumps(self.dummy_validator_config[self.agent_name])
         message = TendermintMessage(performative, info=info)  # type: ignore
         self.context.agent_address = message.sender = self.agent_name
@@ -432,6 +435,86 @@ class TestTendermintHandler:
         log_message = self.handler.LogMessages.collected_config_info.value
         assert log_message in caplog.text
 
+    @pytest.mark.parametrize("registered", (True, False))
+    @pytest.mark.parametrize(
+        "performative",
+        (
+            TendermintMessage.Performative.RECOVERY_PARAMS,
+            TendermintMessage.Performative.GET_RECOVERY_PARAMS,
+        ),
+    )
+    def test_recovery_params(
+        self,
+        registered: bool,
+        performative: TendermintMessage.Performative,
+        caplog: LogCaptureFixture,
+    ) -> None:
+        """Test handle response for recovery parameters."""
+        if performative == TendermintMessage.Performative.GET_RECOVERY_PARAMS:
+            self.context.state.tm_recovery_params = TendermintRecoveryParams(
+                "DummyRound"
+            )
+            message = TendermintMessage(performative)  # type: ignore
+            log_message = self.handler.LogMessages.sending_request_response.value
+        elif performative == TendermintMessage.Performative.RECOVERY_PARAMS:
+            params = json.dumps(
+                asdict(TendermintRecoveryParams(DummyRound.auto_round_id()))
+            )
+            message = TendermintMessage(performative, params=params)  # type: ignore
+            log_message = self.handler.LogMessages.collected_params.value
+        else:
+            raise AssertionError(
+                f"Invalid performative {performative} for `test_recovery_params`."
+            )
+
+        self.context.agent_address = message.sender = self.agent_name
+        registered_addresses = (
+            {self.agent_name: {"dummy": "value"}} if registered else {}
+        )
+
+        with self.mocked_registered_addresses(registered_addresses):
+            self.handler.handle(message)
+
+        if not registered:
+            log_message = self.handler.LogMessages.not_in_registered_addresses.value
+
+        assert log_message in caplog.text
+
+    @pytest.mark.parametrize(
+        "side_effect, expected_exception",
+        (
+            (
+                json.decoder.JSONDecodeError("", "", 0),
+                ": line 1 column 1 (char 0)",
+            ),
+            (
+                {"not a dict"},
+                "argument after ** must be a mapping, not str",
+            ),
+        ),
+    )
+    def test_recovery_params_error(
+        self,
+        side_effect: Any,
+        expected_exception: str,
+        caplog: LogCaptureFixture,
+    ) -> None:
+        """Test handle response for recovery parameters."""
+        message = TendermintMessage(
+            TendermintMessage.Performative.RECOVERY_PARAMS, params=MagicMock()  # type: ignore
+        )
+
+        self.context.agent_address = message.sender = self.agent_name
+        registered_addresses = {self.agent_name: {"dummy": "value"}}
+        with self.mocked_registered_addresses(registered_addresses), mock.patch.object(
+            json, "loads", side_effect=side_effect
+        ):
+            self.handler.handle(message)
+
+        log_message = self.handler.LogMessages.failed_to_parse_params.value
+        assert log_message in caplog.text
+        assert expected_exception in caplog.text
+
     # error
     def test_handle_error(self, caplog: LogCaptureFixture) -> None:
         """Test handle error"""
@@ -464,7 +547,7 @@ class TestTendermintHandler:
     ) -> None:
         """Test performative no recognized"""
         message = self.make_error_message()
-        message._slots.performative = "wacky"
+        message._slots.performative = MagicMock(value="wacky")
         with self.mocked_registered_addresses(self.dummy_validator_config):
             self.handler.handle(message)
         log_message = self.handler.LogMessages.performative_not_recognized.value
@@ -475,7 +558,7 @@ class TestTendermintHandler:
     ) -> None:
         """Test sender not in registered addresses."""
 
-        performative = TendermintMessage.Performative.REQUEST
+        performative = TendermintMessage.Performative.GET_GENESIS_INFO
         message = TendermintMessage(performative)  # type: ignore
         self.context.agent_address = message.sender = "dummy"
         with self.mocked_registered_addresses(self.dummy_validator_config):
