@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -24,18 +24,20 @@
 import builtins
 import json
 import logging
+from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from time import sleep
-from typing import Any, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Type, cast
 from unittest import mock
 from unittest.mock import MagicMock
 
 import pytest
+from aea.exceptions import AEAEnforceError
+from typing_extensions import Literal, TypedDict
 
 from packages.valory.skills.abstract_round_abci.base import (
-    AbciApp,
     AbstractRound,
     BaseSynchronizedData,
     RESET_INDEX_DEFAULT,
@@ -46,11 +48,26 @@ from packages.valory.skills.abstract_round_abci.models import (
     BaseParams,
     BenchmarkTool,
     DEFAULT_BACKOFF_FACTOR,
+    GenesisBlock,
+    GenesisConfig,
+    GenesisConsensusParams,
+    GenesisEvidence,
+    GenesisValidator,
     NUMBER_OF_RETRIES,
     Requests,
-    SharedState,
+)
+from packages.valory.skills.abstract_round_abci.models import (
+    SharedState as BaseSharedState,
+)
+from packages.valory.skills.abstract_round_abci.models import (
+    TendermintRecoveryParams,
+    _MetaSharedState,
+    check_type,
 )
 from packages.valory.skills.abstract_round_abci.test_tools.abci_app import AbciAppTest
+from packages.valory.skills.abstract_round_abci.tests.conftest import (
+    irrelevant_genesis_config,
+)
 
 
 BASE_DUMMY_SPECS_CONFIG = dict(
@@ -59,8 +76,8 @@ BASE_DUMMY_SPECS_CONFIG = dict(
     url="http://dummy",
     api_id="api_id",
     method="GET",
-    headers=[["Dummy-Header", "dummy_value"]],
-    parameters=[["Dummy-Param", "dummy_param"]],
+    headers=[("Dummy-Header", "dummy_value")],
+    parameters=[("Dummy-Param", "dummy_param")],
 )
 
 
@@ -91,7 +108,10 @@ class TestApiSpecsModel:
         """Test initialization."""
 
         # test ensure method.
-        with pytest.raises(ValueError, match="Value for url is required by ApiSpecs"):
+        with pytest.raises(
+            AEAEnforceError,
+            match="'url' of type '<class 'str'>' required, but it is not set in `models.params.args` of `skill.yaml` of",
+        ):
             _ = ApiSpecs(
                 name="dummy",
                 skill_context=MagicMock(),
@@ -104,8 +124,8 @@ class TestApiSpecsModel:
         assert self.api_specs.url == "http://dummy"
         assert self.api_specs.api_id == "api_id"
         assert self.api_specs.method == "GET"
-        assert self.api_specs.headers == [["Dummy-Header", "dummy_value"]]
-        assert self.api_specs.parameters == [["Dummy-Param", "dummy_param"]]
+        assert self.api_specs.headers == [("Dummy-Header", "dummy_value")]
+        assert self.api_specs.parameters == [("Dummy-Param", "dummy_param")]
         assert self.api_specs.response_info.response_key == "value"
         assert self.api_specs.response_info.response_index == 0
         assert self.api_specs.response_info.response_type == "float"
@@ -146,8 +166,8 @@ class TestApiSpecsModel:
         actual_specs = {
             "url": "http://dummy",
             "method": "GET",
-            "headers": [["Dummy-Header", "dummy_value"]],
-            "parameters": [["Dummy-Param", "dummy_param"]],
+            "headers": [("Dummy-Header", "dummy_value")],
+            "parameters": [("Dummy-Param", "dummy_param")],
         }
 
         specs = self.api_specs.get_spec()
@@ -165,7 +185,6 @@ class TestApiSpecsModel:
                     response_type="float",
                     error_key=None,
                     error_index=None,
-                    error_type=None,
                     error_data=None,
                 ),
                 MagicMock(body=b'{"value": "10.232"}'),
@@ -194,7 +213,6 @@ class TestApiSpecsModel:
                     **BASE_DUMMY_SPECS_CONFIG,
                     response_key="test:response:key",
                     response_index=2,
-                    response_type=None,
                     error_key="error:key",
                     error_index=3,
                     error_type="str",
@@ -209,7 +227,6 @@ class TestApiSpecsModel:
                     **BASE_DUMMY_SPECS_CONFIG,
                     response_key="test:response:key",
                     response_index=2,
-                    response_type=None,
                     error_key="error:key",
                     error_index=3,
                     error_type="str",
@@ -219,7 +236,7 @@ class TestApiSpecsModel:
                     # the null will raise `TypeError` and we test that it is handled
                     body=b'{"test": {"response": {"key": ["does_not_matter", "does_not_matter", null]}}}'
                 ),
-                None,
+                "None",
                 None,
             ),
             (
@@ -227,7 +244,6 @@ class TestApiSpecsModel:
                     **BASE_DUMMY_SPECS_CONFIG,
                     response_key="test:response:key",
                     response_index=2,  # this will raise `IndexError` and we test that it is handled
-                    response_type=None,
                     error_key="error:key",
                     error_index=3,
                     error_type="str",
@@ -244,7 +260,6 @@ class TestApiSpecsModel:
                     **BASE_DUMMY_SPECS_CONFIG,
                     response_key="test:response:key",  # this will raise `KeyError` and we test that it is handled
                     response_index=2,
-                    response_type=None,
                     error_key="error:key",
                     error_index=3,
                     error_type="str",
@@ -261,7 +276,6 @@ class TestApiSpecsModel:
                     **BASE_DUMMY_SPECS_CONFIG,
                     response_key="test:response:key",
                     response_index=2,
-                    response_type=None,
                     error_key="error:key",
                     error_index=3,
                     error_type="str",
@@ -287,57 +301,65 @@ class TestApiSpecsModel:
         api_specs = ApiSpecs(**api_specs_config)
         actual = api_specs.process_response(message)
         assert actual == expected_res
-        response_type = api_specs_config["response_type"]
+        response_type = api_specs_config.get("response_type", None)
         if response_type is not None:
             assert type(actual) == getattr(builtins, response_type)
         assert api_specs.response_info.error_data == expected_error
 
+    def test_attribute_manipulation(self) -> None:
+        """Test manipulating the attributes."""
+        with pytest.raises(AttributeError, match="This object is frozen!"):
+            del self.api_specs.url
+
+        with pytest.raises(AttributeError, match="This object is frozen!"):
+            self.api_specs.url = ""
+
+        self.api_specs.__dict__["_frozen"] = False
+        self.api_specs.url = ""
+        del self.api_specs.url
+
 
 class ConcreteRound(AbstractRound):
     """A ConcreteRoundA for testing purposes."""
 
     synchronized_data_class = MagicMock()
     payload_attribute = MagicMock()
-    allowed_tx_type = MagicMock()
+    payload_class = MagicMock()
 
     def end_block(self) -> Optional[Tuple[BaseSynchronizedData, Enum]]:
         """Handle the end of the block."""
 
 
+class SharedState(BaseSharedState):
+    """Shared State for testing purposes."""
+
+    abci_app_cls = AbciAppTest
+
+
 class TestSharedState:
     """Test SharedState(Model) class."""
 
-    @mock.patch.object(SharedState, "_process_abci_app_cls")
     def test_initialization(self, *_: Any) -> None:
         """Test the initialization of the shared state."""
-        SharedState(abci_app_cls=MagicMock(), name="", skill_context=MagicMock())
+        SharedState(name="", skill_context=MagicMock())
 
-    @mock.patch.object(SharedState, "_process_abci_app_cls")
     def test_setup(self, *_: Any) -> None:
         """Test setup method."""
-        shared_state = SharedState(
-            abci_app_cls=MagicMock, name="", skill_context=MagicMock()
-        )
+        shared_state = SharedState(name="", skill_context=MagicMock())
         shared_state.context.params.setup_params = {"test": []}
         shared_state.context.params.consensus_params = MagicMock()
         shared_state.setup()
 
-    @mock.patch.object(SharedState, "_process_abci_app_cls")
     def test_synchronized_data_negative_not_available(self, *_: Any) -> None:
         """Test 'synchronized_data' property getter, negative case (not available)."""
-        shared_state = SharedState(
-            abci_app_cls=AbciAppTest, name="", skill_context=MagicMock()
-        )
+        shared_state = SharedState(name="", skill_context=MagicMock())
         with mock.patch.object(shared_state.context, "params"):
             with pytest.raises(ValueError, match="round sequence not available"):
                 shared_state.synchronized_data
 
-    @mock.patch.object(SharedState, "_process_abci_app_cls")
     def test_synchronized_data_positive(self, *_: Any) -> None:
         """Test 'synchronized_data' property getter, negative case (not available)."""
-        shared_state = SharedState(
-            abci_app_cls=AbciAppTest, name="", skill_context=MagicMock()
-        )
+        shared_state = SharedState(name="", skill_context=MagicMock())
         shared_state.context.params.setup_params = {"test": []}
         shared_state.context.params.consensus_params = MagicMock()
         shared_state.setup()
@@ -346,9 +368,7 @@ class TestSharedState:
 
     def test_synchronized_data_db(self, *_: Any) -> None:
         """Test 'synchronized_data' AbciAppDB."""
-        shared_state = SharedState(
-            abci_app_cls=AbciAppTest, name="", skill_context=MagicMock()
-        )
+        shared_state = SharedState(name="", skill_context=MagicMock())
         with mock.patch.object(shared_state.context, "params") as mock_params:
             mock_params.setup_params = {
                 "safe_contract_address": ["0xsafe"],
@@ -364,46 +384,54 @@ class TestSharedState:
                 == "0xoracle"
             )
 
-    def test_process_abci_app_cls_negative_not_a_class(self) -> None:
-        """Test '_process_abci_app_cls', negative case (not a class)."""
-        mock_obj = MagicMock()
-        with pytest.raises(ValueError, match=f"The object {mock_obj} is not a class"):
-            SharedState._process_abci_app_cls(mock_obj)
-
-    def test_process_abci_app_cls_negative_not_subclass_of_abstract_round(
+    @pytest.mark.parametrize(
+        "address_to_acn_deliverable, n_participants, expected",
+        (
+            ({}, 4, None),
+            ({i: "test" for i in range(4)}, 4, "test"),
+            (
+                {i: TendermintRecoveryParams("test") for i in range(4)},
+                4,
+                TendermintRecoveryParams("test"),
+            ),
+            ({1: "test", 2: "non-matching", 3: "test", 4: "test"}, 4, "test"),
+            ({i: "test" for i in range(4)}, 4, "test"),
+            ({1: "no", 2: "result", 3: "matches", 4: ""}, 4, None),
+        ),
+    )
+    def test_get_acn_result(
         self,
+        address_to_acn_deliverable: Dict[str, Any],
+        n_participants: int,
+        expected: Optional[str],
     ) -> None:
-        """Test '_process_abci_app_cls', negative case (not subclass of AbstractRound)."""
-        with pytest.raises(
-            ValueError,
-            match=f"The class {MagicMock} is not an instance of {AbciApp.__module__}.{AbciApp.__name__}",
-        ):
-            SharedState._process_abci_app_cls(MagicMock)
-
-    def test_process_abci_app_cls_positive(self) -> None:
-        """Test '_process_abci_app_cls', positive case."""
-
-        SharedState._process_abci_app_cls(AbciAppTest)
-
-    def test_recovery_params_on_init(self) -> None:
-        """Test that `tm_recovery_params` get initialized correctly."""
+        """Test `get_acn_result`."""
         shared_state = SharedState(
             abci_app_cls=AbciAppTest, name="", skill_context=MagicMock()
         )
+        shared_state.context.params.setup_params = {"test": []}
+        shared_state.setup()
+        shared_state.synchronized_data.update(participants=set(range(n_participants)))
+        shared_state.address_to_acn_deliverable = address_to_acn_deliverable
+        actual = shared_state.get_acn_result()
+
+        assert actual == expected
+
+    def test_recovery_params_on_init(self) -> None:
+        """Test that `tm_recovery_params` get initialized correctly."""
+        shared_state = SharedState(name="", skill_context=MagicMock())
         assert shared_state.tm_recovery_params is not None
         assert shared_state.tm_recovery_params.round_count == ROUND_COUNT_DEFAULT
         assert shared_state.tm_recovery_params.reset_index == RESET_INDEX_DEFAULT
         assert (
             shared_state.tm_recovery_params.reset_from_round
-            == AbciAppTest.initial_round_cls
+            == AbciAppTest.initial_round_cls.auto_round_id()
         )
         assert shared_state.tm_recovery_params.reset_params is None
 
     def test_set_last_reset_params(self) -> None:
         """Test that `last_reset_params` get set correctly."""
-        shared_state = SharedState(
-            abci_app_cls=AbciAppTest, name="", skill_context=MagicMock()
-        )
+        shared_state = SharedState(name="", skill_context=MagicMock())
         test_params = [("genesis_time", "some-time"), ("initial_height", "0")]
         shared_state.last_reset_params = test_params
         assert shared_state.last_reset_params == test_params
@@ -461,14 +489,14 @@ def test_requests_model_initialization() -> None:
 
 def test_base_params_model_initialization() -> None:
     """Test initialization of the 'BaseParams(Model)' class."""
-    BaseParams(
+    kwargs = dict(
         name="",
         skill_context=MagicMock(),
         setup={},
         consensus=dict(max_participants=1),
         tendermint_url="",
         max_healthcheck=1,
-        round_timeout_seconds=1,
+        round_timeout_seconds=1.0,
         sleep_time=1,
         retry_timeout=1,
         retry_attempts=1,
@@ -482,5 +510,199 @@ def test_base_params_model_initialization() -> None:
         tendermint_check_sleep_delay=3,
         tendermint_max_retries=5,
         cleanup_history_depth=0,
-        genesis_config={"voting_power": "0"},
+        genesis_config=irrelevant_genesis_config,
+        cleanup_history_depth_current=None,
+        request_timeout=0.0,
+        request_retry_delay=0.0,
+        tx_timeout=0.0,
+        max_attempts=0,
+        on_chain_service_id=None,
+        share_tm_config_on_startup=False,
+        tendermint_p2p_url="",
     )
+    bp = BaseParams(**kwargs)
+
+    with pytest.raises(AttributeError, match="This object is frozen!"):
+        bp.request_timeout = 0.1
+
+    with pytest.raises(AttributeError, match="This object is frozen!"):
+        del bp.request_timeout
+
+    bp.__dict__["_frozen"] = False
+    del bp.request_timeout
+
+    assert getattr(bp, "request_timeout", None) is None
+
+    with pytest.raises(
+        TypeError,
+        match="Value `b` in `setup` set in `models.params.args` of `skill.yaml` of .* is not a list",
+    ):
+        kwargs["setup"] = {"a": "b"}
+        BaseParams(**kwargs)
+
+
+def test_genesis_block() -> None:
+    """Test genesis block methods."""
+    json = {"max_bytes": "a", "max_gas": "b", "time_iota_ms": "c"}
+    gb = GenesisBlock(**json)
+    assert gb.to_json() == json
+
+    with pytest.raises(TypeError, match="Error in field 'max_bytes'. Expected type .*"):
+        json["max_bytes"] = 0  # type: ignore
+        GenesisBlock(**json)
+
+
+def test_genesis_evidence() -> None:
+    """Test genesis evidence methods."""
+    json = {"max_age_num_blocks": "a", "max_age_duration": "b", "max_bytes": "c"}
+    ge = GenesisEvidence(**json)
+    assert ge.to_json() == json
+
+
+def test_genesis_validator() -> None:
+    """Test genesis validator methods."""
+    json = {"pub_key_types": ["a", "b"]}
+    ge = GenesisValidator(pub_key_types=tuple(json["pub_key_types"]))
+    assert ge.to_json() == json
+
+    with pytest.raises(
+        TypeError, match="Error in field 'pub_key_types'. Expected type .*"
+    ):
+        GenesisValidator(**json)  # type: ignore
+
+
+def test_genesis_consensus_params() -> None:
+    """Test genesis consensus params methods."""
+    consensus_params = cast(Dict, irrelevant_genesis_config["consensus_params"])
+    gcp = GenesisConsensusParams.from_json_dict(consensus_params)
+    assert gcp.to_json() == consensus_params
+
+
+def test_genesis_config() -> None:
+    """Test genesis config methods."""
+    gcp = GenesisConfig.from_json_dict(irrelevant_genesis_config)
+    assert gcp.to_json() == irrelevant_genesis_config
+
+
+def test_meta_shared_state_when_instance_not_subclass_of_shared_state() -> None:
+    """Test instantiation of meta class when instance not a subclass of shared state."""
+
+    class MySharedState(metaclass=_MetaSharedState):
+        pass
+
+
+def test_shared_state_instantiation_without_attributes_raises_error() -> None:
+    """Test that definition of concrete subclass of SharedState without attributes raises error."""
+    with pytest.raises(AttributeError, match="'abci_app_cls' not set on .*"):
+
+        class MySharedState(BaseSharedState):
+            pass
+
+    with pytest.raises(AttributeError, match="The object `None` is not a class"):
+
+        class MySharedStateB(BaseSharedState):
+            abci_app_cls = None  # type: ignore
+
+    with pytest.raises(
+        AttributeError,
+        match="The class <class 'unittest.mock.MagicMock'> is not an instance of packages.valory.skills.abstract_round_abci.base.AbciApp",
+    ):
+
+        class MySharedStateC(BaseSharedState):
+            abci_app_cls = MagicMock
+
+
+@dataclass
+class A:
+    """Class for testing."""
+
+    value: int
+
+
+@dataclass
+class B:
+    """Class for testing."""
+
+    value: str
+
+
+class C(TypedDict):
+    """Class for testing."""
+
+    name: str
+    year: int
+
+
+class D(TypedDict, total=False):
+    """Class for testing."""
+
+    name: str
+    year: int
+
+
+testdata_positive = [
+    ("test_arg", 1, int),
+    ("test_arg", "1", str),
+    ("test_arg", True, bool),
+    ("test_arg", 1, Optional[int]),
+    ("test_arg", None, Optional[int]),
+    ("test_arg", "1", Optional[str]),
+    ("test_arg", None, Optional[str]),
+    ("test_arg", None, Optional[bool]),
+    ("test_arg", None, Optional[List[int]]),
+    ("test_arg", [], Optional[List[int]]),
+    ("test_arg", [1], Optional[List[int]]),
+    ("test_arg", {"str": 1}, Optional[Dict[str, int]]),
+    ("test_arg", {"str": A(1)}, Dict[str, A]),
+    ("test_arg", [("1", "2")], List[Tuple[str, str]]),
+    ("test_arg", [1], List[Optional[int]]),
+    ("test_arg", [1, None], List[Optional[int]]),
+    ("test_arg", A, Type[A]),
+    ("test_arg", A, Optional[Type[A]]),
+    ("test_arg", None, Optional[Type[A]]),
+    ("test_arg", MagicMock(), Optional[Type[A]]),  # any type allowed
+    ("test_arg", {"name": "str", "year": 1}, C),
+    ("test_arg", 42, Literal[42]),
+    ("test_arg", {"name": "str"}, D),
+]
+
+
+@pytest.mark.parametrize("name,value,type_hint", testdata_positive)
+def test_type_check_positive(name: str, value: Any, type_hint: Any) -> None:
+    """Test the type check mixin."""
+
+    check_type(name, value, type_hint)
+
+
+testdata_negative = [
+    ("test_arg", "1", int),
+    ("test_arg", 1, str),
+    ("test_arg", None, bool),
+    ("test_arg", "1", Optional[int]),
+    ("test_arg", 1, Optional[str]),
+    ("test_arg", 1, Optional[bool]),
+    ("test_arg", ["1"], Optional[List[int]]),
+    ("test_arg", {"str": "1"}, Optional[Dict[str, int]]),
+    ("test_arg", {1: 1}, Optional[Dict[str, int]]),
+    ("test_arg", {"str": B("1")}, Dict[str, A]),
+    ("test_arg", [()], List[Tuple[str, str]]),
+    ("test_arg", [("1",)], List[Tuple[str, str]]),
+    ("test_arg", [("1", 1)], List[Tuple[str, str]]),
+    ("test_arg", [("1", 1, "1")], List[Tuple[str, ...]]),
+    ("test_arg", ["1"], List[Optional[int]]),
+    ("test_arg", [1, None, "1"], List[Optional[int]]),
+    ("test_arg", B, Type[A]),
+    ("test_arg", B, Optional[Type[A]]),
+    ("test_arg", {"name": "str", "year": "1"}, C),
+    ("test_arg", 41, Literal[42]),
+    ("test_arg", C({"name": "str", "year": 1}), A),
+    ("test_arg", {"name": "str"}, C),
+]
+
+
+@pytest.mark.parametrize("name,value,type_hint", testdata_negative)
+def test_type_check_negative(name: str, value: Any, type_hint: Any) -> None:
+    """Test the type check mixin."""
+
+    with pytest.raises(TypeError):
+        check_type(name, value, type_hint)
