@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -44,7 +44,7 @@ class TestAbciConsensusBehaviour(AbstractRoundBehaviour):
     """This behaviour manages the consensus stages for the simple abci app."""
 
     initial_behaviour_cls = DummyBehaviour
-    abci_app_cls = TestAbciApp  # type: ignore
-    behaviours: Set[Type[DummyBehaviour]] = {  # type: ignore
+    abci_app_cls = TestAbciApp
+    behaviours: Set[Type[BaseBehaviour]] = {
         DummyBehaviour,  # type: ignore
     }
