@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ------------------------------------------------------------------------------
 #
-#   Copyright 2021-2022 Valory AG
+#   Copyright 2021-2023 Valory AG
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,23 +18,12 @@
 # ------------------------------------------------------------------------------
 
 """This module contains the transaction payloads for the test_abci app."""
-from abc import ABC
-from enum import Enum
+
+from dataclasses import dataclass
 
 from packages.valory.skills.abstract_round_abci.base import BaseTxPayload
 
 
-class TransactionType(Enum):
-    """Enumeration of transaction types."""
-
-    DUMMY = "dummy"
-
-    def __str__(self) -> str:
-        """Get the string value of the transaction type."""
-        return self.value
-
-
-class DummyPayload(BaseTxPayload, ABC):
-    """Represent a transaction payload of type 'registration'."""
-
-    transaction_type = TransactionType.DUMMY
+@dataclass(frozen=True)
+class DummyPayload(BaseTxPayload):
+    """Represent a transaction payload of type 'dummy payload'."""
