@@ -217,8 +217,8 @@ class FailedRound(DegenerateRound, ABC):
 class CollectSignatureRound(CollectDifferentUntilThresholdRound):
     """A round in which agents sign the transaction"""
 
-    allowed_tx_type = SignaturePayload.transaction_type
-    payload_attribute = get_name(SignaturePayload.signature)
+    payload_class = SignaturePayload
+    payload_attribute = "signature"
     synchronized_data_class = SynchronizedData
     done_event = Event.DONE
     no_majority_event = Event.NO_MAJORITY
@@ -228,8 +228,8 @@ class CollectSignatureRound(CollectDifferentUntilThresholdRound):
 class FinalizationRound(OnlyKeeperSendsRound):
     """A round that represents transaction signing has finished"""
 
-    allowed_tx_type = FinalizationTxPayload.transaction_type
-    payload_attribute = get_name(FinalizationTxPayload.tx_data)
+    payload_class = FinalizationTxPayload
+    payload_attribute = "tx_data"
     synchronized_data_class = SynchronizedData
 
     def end_block(
@@ -296,8 +296,8 @@ class FinalizationRound(OnlyKeeperSendsRound):
 class RandomnessTransactionSubmissionRound(CollectSameUntilThresholdRound):
     """A round for generating randomness"""
 
-    allowed_tx_type = RandomnessPayload.transaction_type
-    payload_attribute = get_name(RandomnessPayload.randomness)
+    payload_class = RandomnessPayload
+    payload_attribute = "randomness"
     synchronized_data_class = SynchronizedData
     done_event = Event.DONE
     no_majority_event = Event.NO_MAJORITY
@@ -308,8 +308,8 @@ class RandomnessTransactionSubmissionRound(CollectSameUntilThresholdRound):
 class SelectKeeperTransactionSubmissionRoundA(CollectSameUntilThresholdRound):
     """A round in which a keeper is selected for transaction submission"""
 
-    allowed_tx_type = SelectKeeperPayload.transaction_type
-    payload_attribute = get_name(SelectKeeperPayload.keepers)
+    payload_class = SelectKeeperPayload
+    payload_attribute = "keepers"
     synchronized_data_class = SynchronizedData
     done_event = Event.DONE
     no_majority_event = Event.NO_MAJORITY
@@ -366,8 +366,8 @@ class SelectKeeperTransactionSubmissionRoundBAfterTimeout(
 class ValidateTransactionRound(VotingRound):
     """A round in which agents validate the transaction"""
 
-    allowed_tx_type = ValidatePayload.transaction_type
-    payload_attribute = get_name(ValidatePayload.vote)
+    payload_class = ValidatePayload
+    payload_attribute = "vote"
     synchronized_data_class = SynchronizedData
     done_event = Event.DONE
     negative_event = Event.NEGATIVE
@@ -416,8 +416,8 @@ class ValidateTransactionRound(VotingRound):
 class CheckTransactionHistoryRound(CollectSameUntilThresholdRound):
     """A round in which agents check the transaction history to see if any previous tx has been validated"""
 
-    allowed_tx_type = CheckTransactionHistoryPayload.transaction_type
-    payload_attribute = get_name(CheckTransactionHistoryPayload.verified_res)
+    payload_class = CheckTransactionHistoryPayload
+    payload_attribute = "verified_res"
     synchronized_data_class = SynchronizedData
     collection_key = get_name(SynchronizedData.participant_to_check)
     selection_key = get_name(SynchronizedData.most_voted_check_result)
@@ -476,8 +476,8 @@ class CheckLateTxHashesRound(CheckTransactionHistoryRound):
 class SynchronizeLateMessagesRound(CollectNonEmptyUntilThresholdRound):
     """A round in which agents synchronize potentially late arriving messages"""
 
-    allowed_tx_type = SynchronizeLateMessagesPayload.transaction_type
-    payload_attribute = get_name(SynchronizeLateMessagesPayload.tx_hashes)
+    payload_class = SynchronizeLateMessagesPayload
+    payload_attribute = "tx_hashes"
     synchronized_data_class = SynchronizedData
     done_event = Event.DONE
     no_majority_event = Event.NO_MAJORITY
@@ -513,8 +513,8 @@ class FinishedTransactionSubmissionRound(DegenerateRound, ABC):
 class ResetRound(CollectSameUntilThresholdRound):
     """A round that represents the reset of a period"""
 
-    allowed_tx_type = ResetPayload.transaction_type
-    payload_attribute = get_name(ResetPayload.period_count)
+    payload_class = ResetPayload
+    payload_attribute = "period_count"
     synchronized_data_class = SynchronizedData
 
     def end_block(self) -> Optional[Tuple[BaseSynchronizedData, Event]]:
