@@ -137,3 +137,76 @@ skip_docker_tests  # unused variable (autonomy/test_tools/docker/base.py:40)
 run_service_locally  # unused function (autonomy.cli.develop.run_service_locally)
 lock_packages  # unused function (autonomy/cli/packages.py:33)
 PathArgument  # unused class (autonomy/cli/utils/click_utils.py:91)
+protocol  # unused function (autonomy/cli/develop.py:101)
+local  # unused variable (autonomy/chain/config.py:71)
+custom  # unused variable (autonomy/chain/config.py:77)
+goerli  # unused variable (autonomy/chain/config.py:83)
+ethereum  # unused variable (autonomy/chain/config.py:89)
+agent_registry  # unused variable (autonomy/chain/config.py:116)
+service_manager  # unused variable (autonomy/chain/config.py:138)
+gnosis_safe_proxy_factory  # unused variable (autonomy/chain/config.py:163)
+COMPONENT_REGISTRY_ABI_FILENAME  # unused variable (autonomy/chain/constants.py:38)
+AGENT_REGISTRY_ABI_FILENAME  # unused variable (autonomy/chain/constants.py:39)
+REGISTRIES_MANAGER_ABI_FILENAME  # unused variable (autonomy/chain/constants.py:40)
+SERVICE_MANAGER_ABI_FILENAME  # unused variable (autonomy/chain/constants.py:41)
+GNOSIS_SAFE_MULTISIG_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:51)
+GNOSIS_SAFE_MULTISIG_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:60)
+GNOSIS_SAFE_MULTISIG_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:68)
+GNOSIS_SAFE_MULTISIG_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:76)
+COMPONENT_REGISTRY_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:37)
+AGENT_REGISTRY_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:38)
+REGISTRIES_MANAGER_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:39)
+SERVICE_MANAGER_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:40)
+SERVICE_REGISTRY_ADDRESS_LOCAL  # unused variable (autonomy/chain/constants.py:43)
+COMPONENT_REGISTRY_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:45)
+AGENT_REGISTRY_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:46)
+REGISTRIES_MANAGER_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:47)
+SERVICE_MANAGER_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:48)
+SERVICE_REGISTRY_ADDRESS_ETHEREUM  # unused variable (autonomy/chain/constants.py:49)
+COMPONENT_REGISTRY_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:52)
+AGENT_REGISTRY_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:53)
+REGISTRIES_MANAGER_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:54)
+SERVICE_MANAGER_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:55)
+SERVICE_REGISTRY_ADDRESS_GOERLI  # unused variable (autonomy/chain/constants.py:56)
+COMPONENT_REGISTRY_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:59)
+AGENT_REGISTRY_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:60)
+REGISTRIES_MANAGER_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:61)
+SERVICE_MANAGER_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:62)
+SERVICE_REGISTRY_ADDRESS_CUSTOM  # unused variable (autonomy/chain/constants.py:63)
+abi_file  # unused variable (open-autonomy/autonomy/chain/config.py:65)
+AGENT_REGISTRY_CONTRACT  # unused variable (open-autonomy/autonomy/chain/constants.py:72)
+SERVICE_MANAGER_CONTRACT  # unused variable (open-autonomy/autonomy/chain/constants.py:74)
+SERVICE_REGISTRY_CONTRACT  # unused variable (open-autonomy/autonomy/chain/constants.py:75)
+ComponentRegistryContract  # unused class (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:39)
+contract_id  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:42)
+get_raw_transaction  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:50)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:52)
+get_raw_message  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:57)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:59)
+get_state  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:64)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/component_registry/contract.py:66)
+RegistriesManagerContract  # unused class (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:39)
+contract_id  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:42)
+get_raw_transaction  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:50)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:52)
+get_raw_message  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:57)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:59)
+get_state  # unused method (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:64)
+kwargs  # unused variable (open-autonomy/autonomy/data/packages/valory/contracts/registries_manager/contract.py:66)
+path_to_contract  # unused variable (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:54)
+ledger_identifier  # unused variable (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:55)
+finish_contract_deployment  # unused method (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:57)
+_deploy_contract  # unused method (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:62)
+deployer_crypto  # unused variable (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:67)
+gas  # unused variable (open-autonomy/autonomy/data/contracts/service_registry/tests/test_contract.py:68)
+connection  # unused function (open-autonomy/autonomy/cli/mint.py:127)
+skill  # unused function (open-autonomy/autonomy/cli/mint.py:155)
+AgentRegistryContract  # unused class (open-autonomy/autonomy/data/contracts/agent_registry/contract.py:38)
+default_account  # unused attribute (autonomy/cli/helpers/chain.py:85:)
+activate  # unused function (autonomy/cli/service.py:48)
+register  # unused function (autonomy/cli/service.py:78)
+deploy  # unused function (autonomy/cli/service.py:126)
+DEPLOYED  # unused variable (open-autonomy/autonomy/chain/base.py:53)
+TERMINATED_BONDED  # unused variable (open-autonomy/autonomy/chain/base.py:54)
+contract  # unused function (open-autonomy/autonomy/cli/mint.py:106)
+directory  # unused attribute (open-autonomy/autonomy/cli/helpers/analyse.py:43)
